@@ -44,7 +44,18 @@ define ([
     prototypes
 ) {
 
-    var heldObject = null;
+    var heldObject = null,
+        dbrowserState = {
+            "obj": null,  // the dbrowser object itself
+            "set": null,  // the dataset (array) we are browsing
+            "pos": null   // the current position within that array
+        },
+        drowselectState = {
+            "obj": null,  // the drowselect object itself
+            "set": null,  // the dataset (array) we are selecting from
+            "pos": null   // the current position within that array
+        };
+
 
     return {
 
@@ -182,7 +193,30 @@ define ([
         // convert null to empty array
         forceArray: function (arr) {
             return (arr === null) ? [] : arr;
-        }
+        },
+
+        // generate string "n objects" based on array length
+        genObjStr: function (len) {
+            return (len === 1) ?
+                '1 object' :
+                len + " objects";
+        },
+
+        // reverse-video a row (on/off)
+        reverseVideo: function (row, onoff) {
+            if (onoff === true) {
+                $('#row' + row).css('background-color','#000000');
+                $('#row' + row).css('color','#ffffff');
+            }
+            if (onoff === false) {
+                $('#row' + row).css('background-color','#d0e4fe');
+                $('#row' + row).css('color','#000000');
+            }
+        },
+
+        // dbrowser, drowselect states
+        dbrowserState: dbrowserState,
+        drowselectState: drowselectState
 
     };
 });
